@@ -1042,8 +1042,7 @@ tp_clickfinger_within_distance(struct tp_dispatch *tp,
 	if (!t1 || !t2)
 		return 0;
 
-	if (t1->thumb.state == THUMB_STATE_YES ||
-	    t2->thumb.state == THUMB_STATE_YES)
+	if (tp_thumb_ignored(tp, t1) || tp_thumb_ignored(tp, t2))
 		return 0;
 
 	x = abs(t1->point.x - t2->point.x);
@@ -1098,7 +1097,7 @@ tp_clickfinger_set_button(struct tp_dispatch *tp)
 		if (t->state != TOUCH_BEGIN && t->state != TOUCH_UPDATE)
 			continue;
 
-		if (t->thumb.state == THUMB_STATE_YES)
+		if (tp_thumb_ignored(tp, t))
 			continue;
 
 		if (t->palm.state != PALM_NONE)
