@@ -325,6 +325,10 @@ struct evdev_dispatch_interface {
 	enum libinput_switch_state
 		(*get_switch_state)(struct evdev_dispatch *dispatch,
 				    enum libinput_switch which);
+
+	void (*left_handed_toggle)(struct evdev_dispatch *dispatch,
+				   struct evdev_device *device,
+				   bool left_handed_enabled);
 };
 
 enum evdev_dispatch_type {
@@ -332,6 +336,7 @@ enum evdev_dispatch_type {
 	DISPATCH_TOUCHPAD,
 	DISPATCH_TABLET,
 	DISPATCH_TABLET_PAD,
+	DISPATCH_TOTEM,
 };
 
 struct evdev_dispatch {
@@ -427,6 +432,9 @@ evdev_lid_switch_dispatch_create(struct evdev_device *device);
 struct evdev_dispatch *
 fallback_dispatch_create(struct libinput_device *libinput_device);
 
+struct evdev_dispatch *
+evdev_totem_create(struct evdev_device *device);
+
 bool
 evdev_is_fake_mt_device(struct evdev_device *device);
 
@@ -918,7 +926,6 @@ evdev_phys_rect_to_units(const struct evdev_device *device,
 	return units;
 }
 
-
 static inline void
 evdev_device_init_abs_range_warnings(struct evdev_device *device)
 {
