@@ -250,11 +250,11 @@ quirk_get_name(enum quirk q)
 	case QUIRK_MODEL_SYSTEM76_KUDU:			return "ModelSystem76Kudu";
 	case QUIRK_MODEL_TABLET_MODE_NO_SUSPEND:	return "ModelTabletModeNoSuspend";
 	case QUIRK_MODEL_TABLET_MODE_SWITCH_UNRELIABLE:	return "ModelTabletModeSwitchUnreliable";
-	case QUIRK_MODEL_TABLET_NO_PROXIMITY_OUT:	return "ModelTabletNoProximityOut";
-	case QUIRK_MODEL_TABLET_NO_TILT:		return "ModelTabletNoTilt";
 	case QUIRK_MODEL_TOUCHPAD_VISIBLE_MARKER:	return "ModelTouchpadVisibleMarker";
 	case QUIRK_MODEL_TRACKBALL:			return "ModelTrackball";
 	case QUIRK_MODEL_WACOM_TOUCHPAD:		return "ModelWacomTouchpad";
+	case QUIRK_MODEL_WACOM_ISDV4_PEN:		return "ModelWacomISDV4Pen";
+	case QUIRK_MODEL_DELL_CANVAS_TOTEM:		return "ModelDellCanvasTotem";
 
 	case QUIRK_ATTR_SIZE_HINT:			return "AttrSizeHint";
 	case QUIRK_ATTR_TOUCH_SIZE_RANGE:		return "AttrTouchSizeRange";
@@ -292,7 +292,7 @@ matchflagname(enum match_flags f)
 	default:
 		abort();
 	}
-};
+}
 
 static inline struct property *
 property_new(void)
@@ -312,7 +312,7 @@ property_ref(struct property *p)
 	assert(p->refcount > 0);
 	p->refcount++;
 	return p;
-};
+}
 
 static inline struct property *
 property_unref(struct property *p)
@@ -323,7 +323,7 @@ property_unref(struct property *p)
 	p->refcount--;
 
 	return NULL;
-};
+}
 
 /* Separate call so we can verify that the caller unrefs the property
  * before shutting down the subsystem.
@@ -737,8 +737,8 @@ parse_attr(struct quirks_context *ctx,
 		p->value.s = safe_strdup(value);
 		rc = true;
 	} else if (streq(key, quirk_get_name(QUIRK_ATTR_EVENT_CODE_DISABLE))) {
-		size_t nevents = 32;
-		struct input_event events[nevents];
+		struct input_event events[32];
+		size_t nevents = ARRAY_LENGTH(events);
 		p->id = QUIRK_ATTR_EVENT_CODE_DISABLE;
 		if (!parse_evcode_property(value, events, &nevents) ||
 		    nevents == 0)
@@ -913,8 +913,13 @@ parse_file(struct quirks_context *ctx, const char *path)
 			section = section_new(path, line);
 			list_append(&ctx->sections, &section->link);
 			break;
-		/* entries must start with A-Z */
-		case 'A'...'Z':
+		default:
+			/* entries must start with A-Z */
+			if (line[0] < 'A' && line[0] > 'Z') {
+				qlog_parser(ctx, "%s:%d: Unexpected line %s\n",
+						 path, lineno, line);
+				goto out;
+			}
 			switch (state) {
 			case STATE_SECTION:
 				qlog_parser(ctx, "%s:%d: expected [Section], got %s\n",
@@ -949,10 +954,6 @@ parse_file(struct quirks_context *ctx, const char *path)
 				goto out;
 			}
 			break;
-		default:
-			qlog_parser(ctx, "%s:%d: Unexpected line %s\n",
-					 path, lineno, line);
-			goto out;
 		}
 	}
 
