@@ -32,6 +32,7 @@ struct libinput;
 
 struct libinput_timer {
 	struct libinput *libinput;
+	char *timer_name;
 	struct list link;
 	uint64_t expire; /* in absolute us CLOCK_MONOTONIC */
 	void (*timer_func)(uint64_t now, void *timer_func_data);
@@ -40,9 +41,13 @@ struct libinput_timer {
 
 void
 libinput_timer_init(struct libinput_timer *timer, struct libinput *libinput,
+		    const char *timer_name,
 		    void (*timer_func)(uint64_t now, void *timer_func_data),
 		    void *timer_func_data);
 
+void
+libinput_timer_destroy(struct libinput_timer *timer);
+
 /* Set timer expire time, in absolute us CLOCK_MONOTONIC */
 void
 libinput_timer_set(struct libinput_timer *timer, uint64_t expire);
@@ -66,4 +71,7 @@ libinput_timer_subsys_init(struct libinput *libinput);
 void
 libinput_timer_subsys_destroy(struct libinput *libinput);
 
+void
+libinput_timer_flush(struct libinput *libinput, uint64_t now);
+
 #endif
