@@ -54,6 +54,8 @@
 #include <termios.h>
 #endif
 
+#include <valgrind/valgrind.h>
+
 #include "litest.h"
 #include "litest-int.h"
 #include "libinput-util.h"
@@ -64,8 +66,8 @@
 
 #define UDEV_RULES_D "/run/udev/rules.d"
 #define UDEV_RULE_PREFIX "99-litest-"
-#define UDEV_MODEL_QUIRKS_RULE_FILE UDEV_RULES_D \
-	"/91-litest-model-quirks-REMOVEME-XXXXXX.rules"
+#define UDEV_FUZZ_OVERRIDE_RULE_FILE UDEV_RULES_D \
+	"/91-litest-fuzz-override-REMOVEME-XXXXXX.rules"
 #define UDEV_TEST_DEVICE_RULE_FILE UDEV_RULES_D \
 	"/91-litest-test-device-REMOVEME-XXXXXXX.rules"
 #define UDEV_DEVICE_GROUPS_FILE UDEV_RULES_D \
@@ -116,6 +118,11 @@ litest_backtrace(void)
 	pid_t parent, child;
 	int pipefd[2];
 
+	if (RUNNING_ON_VALGRIND) {
+		litest_log("  Using valgrind, omitting backtrace\n");
+		return;
+	}
+
 	if (pipe(pipefd) == -1)
 		return;
 
@@ -199,6 +206,24 @@ litest_fail_comparison_int(const char *file,
 	abort();
 }
 
+__attribute__((noreturn))
+void
+litest_fail_comparison_double(const char *file,
+			      int line,
+			      const char *func,
+			      const char *operator,
+			      double a,
+			      double b,
+			      const char *astr,
+			      const char *bstr)
+{
+	litest_log("FAILED COMPARISON: %s %s %s\n", astr, operator, bstr);
+	litest_log("Resolved to: %.3f %s %.3f\n", a, operator, b);
+	litest_log("in %s() (%s:%d)\n", func, file, line);
+	litest_backtrace();
+	abort();
+}
+
 __attribute__((noreturn))
 void
 litest_fail_comparison_ptr(const char *file,
@@ -356,6 +381,25 @@ static struct suite *
 get_suite(const char *name)
 {
 	struct suite *s;
+	/* this is the list meson calls, ensure we don't miss out on tests */
+	const char * allowed_suites[] = {
+		"config:", "context:", "device:", "events:", "gestures:",
+		"keyboard:", "lid:", "log:", "misc:", "pad:", "path:",
+		"pointer:", "quirks:", "switch:", "tablet:", "tablet-mode:",
+		"tap:", "timer:", "totem:", "touch:", "touchpad:",
+		"trackball:", "trackpoint:", "udev:",
+	};
+	const char **allowed;
+	bool found = false;
+
+	ARRAY_FOR_EACH(allowed_suites, allowed) {
+		if (strneq(name, *allowed, strlen(*allowed))) {
+			found = true;
+			break;
+		}
+	}
+	if (!found)
+		litest_abort_msg("Suite name '%s' is not allowed\n", name);
 
 	list_for_each(s, &all_tests, node) {
 		if (streq(s->name, name))
@@ -375,8 +419,8 @@ static void
 litest_add_tcase(const char *suite_name,
 		 const char *funcname,
 		 void *func,
-		 enum litest_device_feature required,
-		 enum litest_device_feature excluded,
+		 int64_t required,
+		 int64_t excluded,
 		 const struct range *range)
 {
 	struct suite *suite;
@@ -490,8 +534,8 @@ void
 _litest_add(const char *name,
 	    const char *funcname,
 	    void *func,
-	    enum litest_device_feature required,
-	    enum litest_device_feature excluded)
+	    int64_t required,
+	    int64_t excluded)
 {
 	_litest_add_ranged(name,
 			   funcname,
@@ -505,8 +549,8 @@ void
 _litest_add_ranged(const char *name,
 		   const char *funcname,
 		   void *func,
-		   enum litest_device_feature required,
-		   enum litest_device_feature excluded,
+		   int64_t required,
+		   int64_t excluded,
 		   const struct range *range)
 {
 	litest_add_tcase(name, funcname, func, required, excluded, range);
@@ -623,7 +667,7 @@ litest_log_handler(struct libinput *libinput,
 		/* valgrind is too slow and some of our offsets are too
 		 * short, don't abort if during a valgrind run we get a
 		 * negative offset */
-		if (!getenv("USING_VALGRIND") ||
+		if ((!RUNNING_ON_VALGRIND && !in_debugger) ||
 		    !strstr(format, "offset negative"))
 		litest_abort_msg("libinput bug triggered, aborting.\n");
 	}
@@ -633,24 +677,61 @@ litest_log_handler(struct libinput *libinput,
 	}
 }
 
-static char *
-litest_init_device_udev_rules(struct litest_test_device *dev);
+static void
+litest_init_device_udev_rules(struct litest_test_device *dev, FILE *f)
+{
+	const struct key_value_str *kv;
+	static int count;
+
+	if (dev->udev_properties[0].key == NULL)
+		return;
+
+	count++;
+
+	fprintf(f, "# %s\n", dev->shortname);
+	fprintf(f, "ACTION==\"remove\", GOTO=\"rule%d_end\"\n", count);
+	fprintf(f, "KERNEL!=\"event*\", GOTO=\"rule%d_end\"\n", count);
+
+	fprintf(f, "ATTRS{name}==\"litest %s*\"", dev->name);
+
+	kv = dev->udev_properties;
+	while (kv->key) {
+		fprintf(f, ", \\\n\tENV{%s}=\"%s\"", kv->key, kv->value);
+		kv++;
+	}
+
+	fprintf(f, "\n");
+	fprintf(f, "LABEL=\"rule%d_end\"\n\n", count);;
+}
 
 static void
 litest_init_all_device_udev_rules(struct list *created_files)
 {
+	struct created_file *file = zalloc(sizeof(*file));
 	struct litest_test_device *dev;
+	char *path = NULL;
+	FILE *f;
+	int rc;
+	int fd;
 
-	list_for_each(dev, &devices, node) {
-		char *udev_file;
+	rc = xasprintf(&path,
+		      "%s/%s-XXXXXX.rules",
+		      UDEV_RULES_D,
+		      UDEV_RULE_PREFIX);
+	litest_assert_int_gt(rc, 0);
 
-		udev_file = litest_init_device_udev_rules(dev);
-		if (udev_file) {
-			struct created_file *file = zalloc(sizeof(*file));
-			file->path = udev_file;
-			list_insert(created_files, &file->link);
-		}
-	}
+	fd = mkstemps(path, 6);
+	litest_assert_int_ne(fd, -1);
+	f = fdopen(fd, "w");
+	litest_assert_notnull(f);
+
+	list_for_each(dev, &devices, node)
+		litest_init_device_udev_rules(dev, f);
+
+	fclose(f);
+
+	file->path = path;
+	list_insert(created_files, &file->link);
 }
 
 static int
@@ -797,7 +878,7 @@ litest_run_suite(struct list *tests, int which, int max, int error_fd)
 				  s->name,
 				  t->name,
 				  t->devname);
-			litest_assert(sname != NULL);
+			litest_assert_ptr_notnull(sname);
 			n = zalloc(sizeof(*n));
 			n->name = sname;
 			list_insert(&testnames, &n->node);
@@ -806,7 +887,7 @@ litest_run_suite(struct list *tests, int which, int max, int error_fd)
 				  "%s:%s",
 				  t->name,
 				  t->devname);
-			litest_assert(tname != NULL);
+			litest_assert_ptr_notnull(tname);
 			n = zalloc(sizeof(*n));
 			n->name = tname;
 			list_insert(&testnames, &n->node);
@@ -843,11 +924,22 @@ litest_run_suite(struct list *tests, int which, int max, int error_fd)
 
 		trs = srunner_failures(sr);
 		for (int i = 0; i < failed; i++) {
+			char tname[256];
+			char *c = tname;
+
+			/* tr_tcname is in the form "suite:testcase", let's
+			 * convert this to "suite(testcase)" to make
+			 * double-click selection in the terminal a bit
+			 * easier. */
+			snprintf(tname, sizeof(tname), "%s)", tr_tcname(trs[i]));
+			if ((c = index(c, ':')))
+				*c = '(';
+
 			dprintf(error_fd,
-				":: Failure: %s:%d:%s\n",
+				":: Failure: %s:%d: %s\n",
 				tr_lfile(trs[i]),
 				tr_lno(trs[i]),
-				tr_tcname(trs[i]));
+				tname);
 		}
 		free(trs);
 	}
@@ -876,7 +968,7 @@ litest_fork_subtests(struct list *tests, int max_forks)
 		int rc;
 		int pipefd[2];
 
-		rc = pipe2(pipefd, O_NONBLOCK|O_NONBLOCK);
+		rc = pipe2(pipefd, O_NONBLOCK);
 		assert(rc != -1);
 
 		pid = fork();
@@ -898,7 +990,7 @@ litest_fork_subtests(struct list *tests, int max_forks)
 
 	/* parent process only */
 	while (wait(&status) != -1 && errno != ECHILD) {
-		if (WEXITSTATUS(status) != 0)
+		if (!WIFEXITED(status) || WEXITSTATUS(status) != 0)
 			failed = 1;
 	}
 
@@ -1030,7 +1122,7 @@ merge_absinfo(const struct input_absinfo *orig,
 		return NULL;
 
 	abs = zalloc(sz * sizeof(*abs));
-	litest_assert(abs != NULL);
+	litest_assert_ptr_notnull(abs);
 
 	nelem = 0;
 	while (orig[nelem].value != -1) {
@@ -1064,7 +1156,7 @@ merge_events(const int *orig, const int *override)
 		return NULL;
 
 	events = zalloc(sz * sizeof(int));
-	litest_assert(events != NULL);
+	litest_assert_ptr_notnull(events);
 
 	nelem = 0;
 	while (orig[nelem] != -1) {
@@ -1168,8 +1260,8 @@ litest_install_model_quirks(struct list *created_files_list)
 				true);
 	list_insert(created_files_list, &file->link);
 
-	file = litest_copy_file(UDEV_MODEL_QUIRKS_RULE_FILE,
-				LIBINPUT_MODEL_QUIRKS_UDEV_RULES_FILE,
+	file = litest_copy_file(UDEV_FUZZ_OVERRIDE_RULE_FILE,
+				LIBINPUT_FUZZ_OVERRIDE_UDEV_RULES_FILE,
 				warning,
 				true);
 	list_insert(created_files_list, &file->link);
@@ -1342,40 +1434,6 @@ litest_remove_udev_rules(struct list *created_files_list)
 		litest_reload_udev_rules();
 }
 
-static char *
-litest_init_device_udev_rules(struct litest_test_device *dev)
-{
-	int rc;
-	int fd;
-	FILE *f;
-	char *path = NULL;
-	static int count;
-
-	if (!dev->udev_rule)
-		return NULL;
-
-	rc = xasprintf(&path,
-		      "%s/%s%03d-%s-XXXXXX.rules",
-		      UDEV_RULES_D,
-		      UDEV_RULE_PREFIX,
-		      ++count,
-		      dev->shortname);
-	litest_assert_int_eq(rc,
-			     (int)(
-				   strlen(UDEV_RULES_D) +
-				   strlen(UDEV_RULE_PREFIX) +
-				   strlen(dev->shortname) + 18));
-
-	fd = mkstemps(path, 6);
-	litest_assert_int_ne(fd, -1);
-	f = fdopen(fd, "w");
-	litest_assert_notnull(f);
-	litest_assert_int_ge(fputs(dev->udev_rule, f), 0);
-	fclose(f);
-
-	return path;
-}
-
 /**
  * Creates a uinput device but does not add it to a libinput context
  */
@@ -1446,7 +1504,7 @@ litest_create(enum litest_device_type which,
 	free(events);
 
 	path = libevdev_uinput_get_devnode(d->uinput);
-	litest_assert(path != NULL);
+	litest_assert_ptr_notnull(path);
 	fd = open(path, O_RDWR|O_NONBLOCK);
 	litest_assert_int_ne(fd, -1);
 
@@ -1524,15 +1582,15 @@ litest_add_device_with_overrides(struct libinput *libinput,
 			  events_override);
 
 	path = libevdev_uinput_get_devnode(d->uinput);
-	litest_assert(path != NULL);
+	litest_assert_ptr_notnull(path);
 
 	d->libinput = libinput;
 	d->libinput_device = libinput_path_add_device(d->libinput, path);
+	litest_assert_ptr_notnull(d->libinput_device);
 	ud = libinput_device_get_udev_device(d->libinput_device);
 	d->quirks = quirks_fetch_for_device(quirks_context, ud);
 	udev_device_unref(ud);
 
-	litest_assert(d->libinput_device != NULL);
 	libinput_device_ref(d->libinput_device);
 
 	if (d->interface) {
@@ -1698,6 +1756,9 @@ litest_event(struct litest_device *d, unsigned int type,
 {
 	int ret;
 
+	if (!libevdev_has_event_code(d->evdev, type, code))
+		return;
+
 	if (d->skip_ev_syn && type == EV_SYN && code == SYN_REPORT)
 		return;
 
@@ -1773,8 +1834,16 @@ litest_auto_assign_value(struct litest_device *d,
 		break;
 	default:
 		if (!axis_replacement_value(d, axes, ev->code, &value) &&
-		    d->interface->get_axis_default)
-			d->interface->get_axis_default(d, ev->code, &value);
+		    d->interface->get_axis_default) {
+			int error = d->interface->get_axis_default(d,
+								   ev->code,
+								   &value);
+			if (error) {
+				litest_abort_msg("Failed to get default axis value for %s (%d)\n",
+						 libevdev_event_code_get_name(EV_ABS, ev->code),
+						 ev->code);
+			}
+		}
 		break;
 	}
 
@@ -1803,7 +1872,7 @@ slot_start(struct litest_device *d,
 {
 	struct input_event *ev;
 
-	litest_assert(d->ntouches_down >= 0);
+	litest_assert_int_ge(d->ntouches_down, 0);
 	d->ntouches_down++;
 
 	send_btntool(d, !touching);
@@ -2135,22 +2204,34 @@ auto_assign_tablet_value(struct litest_device *d,
 			 int x, int y,
 			 struct axis_replacement *axes)
 {
+	static int tracking_id;
 	int value = ev->value;
 
 	if (value != LITEST_AUTO_ASSIGN || ev->type != EV_ABS)
 		return value;
 
 	switch (ev->code) {
+	case ABS_MT_TRACKING_ID:
+		value = ++tracking_id;
+		break;
 	case ABS_X:
+	case ABS_MT_POSITION_X:
 		value = litest_scale(d, ABS_X, x);
 		break;
 	case ABS_Y:
+	case ABS_MT_POSITION_Y:
 		value = litest_scale(d, ABS_Y, y);
 		break;
 	default:
 		if (!axis_replacement_value(d, axes, ev->code, &value) &&
-		    d->interface->get_axis_default)
-			d->interface->get_axis_default(d, ev->code, &value);
+		    d->interface->get_axis_default) {
+			int error = d->interface->get_axis_default(d, ev->code, &value);
+			if (error) {
+				litest_abort_msg("Failed to get default axis value for %s (%d)\n",
+						 libevdev_event_code_get_name(EV_ABS, ev->code),
+						 ev->code);
+			}
+		}
 		break;
 	}
 
@@ -2624,8 +2705,8 @@ litest_wait_for_event_of_type(struct libinput *li, ...)
 	va_start(args, li);
 	type = va_arg(args, int);
 	while ((int)type != -1) {
-		litest_assert(type > 0);
-		litest_assert(ntypes < ARRAY_LENGTH(types));
+		litest_assert_int_gt(type, 0U);
+		litest_assert_int_lt(ntypes, ARRAY_LENGTH(types));
 		types[ntypes++] = type;
 		type = va_arg(args, int);
 	}
@@ -2683,8 +2764,8 @@ litest_drain_events_of_type(struct libinput *li, ...)
 	va_start(args, li);
 	type = va_arg(args, int);
 	while ((int)type != -1) {
-		litest_assert(type > 0);
-		litest_assert(ntypes < ARRAY_LENGTH(types));
+		litest_assert_int_gt(type, 0U);
+		litest_assert_int_lt(ntypes, ARRAY_LENGTH(types));
 		types[ntypes++] = type;
 		type = va_arg(args, int);
 	}
@@ -2966,7 +3047,7 @@ litest_create_uinput(const char *name,
 	const char *devnode;
 
 	dev = libevdev_new();
-	litest_assert(dev != NULL);
+	litest_assert_ptr_notnull(dev);
 
 	snprintf(buf, sizeof(buf), "litest %s", name);
 	libevdev_set_name(dev, buf);
@@ -3138,7 +3219,7 @@ litest_is_button_event(struct libinput_event *event,
 	struct libinput_event_pointer *ptrev;
 	enum libinput_event_type type = LIBINPUT_EVENT_POINTER_BUTTON;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 	ptrev = libinput_event_get_pointer_event(event);
 	litest_assert_int_eq(libinput_event_pointer_get_button(ptrev),
@@ -3157,7 +3238,7 @@ litest_is_axis_event(struct libinput_event *event,
 	struct libinput_event_pointer *ptrev;
 	enum libinput_event_type type = LIBINPUT_EVENT_POINTER_AXIS;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 	ptrev = libinput_event_get_pointer_event(event);
 	litest_assert(libinput_event_pointer_has_axis(ptrev, axis));
@@ -3176,7 +3257,7 @@ litest_is_motion_event(struct libinput_event *event)
 	enum libinput_event_type type = LIBINPUT_EVENT_POINTER_MOTION;
 	double x, y, ux, uy;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 	ptrev = libinput_event_get_pointer_event(event);
 
@@ -3226,7 +3307,7 @@ litest_is_touch_event(struct libinput_event *event,
 {
 	struct libinput_event_touch *touch;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 
 	if (type == 0)
 		type = libinput_event_get_type(event);
@@ -3256,11 +3337,11 @@ litest_is_keyboard_event(struct libinput_event *event,
 	struct libinput_event_keyboard *kevent;
 	enum libinput_event_type type = LIBINPUT_EVENT_KEYBOARD_KEY;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 
 	kevent = libinput_event_get_keyboard_event(event);
-	litest_assert(kevent != NULL);
+	litest_assert_ptr_notnull(kevent);
 
 	litest_assert_int_eq(libinput_event_keyboard_get_key(kevent), key);
 	litest_assert_int_eq(libinput_event_keyboard_get_key_state(kevent),
@@ -3275,11 +3356,11 @@ litest_is_gesture_event(struct libinput_event *event,
 {
 	struct libinput_event_gesture *gevent;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 
 	gevent = libinput_event_get_gesture_event(event);
-	litest_assert(gevent != NULL);
+	litest_assert_ptr_notnull(gevent);
 
 	if (nfingers != -1)
 		litest_assert_int_eq(libinput_event_gesture_get_finger_count(gevent),
@@ -3293,11 +3374,11 @@ litest_is_tablet_event(struct libinput_event *event,
 {
 	struct libinput_event_tablet_tool *tevent;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 
 	tevent = libinput_event_get_tablet_tool_event(event);
-	litest_assert(tevent != NULL);
+	litest_assert_ptr_notnull(tevent);
 
 	return tevent;
 }
@@ -3341,6 +3422,24 @@ void litest_assert_tablet_proximity_event(struct libinput *li,
 	libinput_event_destroy(event);
 }
 
+void litest_assert_tablet_tip_event(struct libinput *li,
+				    enum libinput_tablet_tool_tip_state state)
+{
+	struct libinput_event *event;
+	struct libinput_event_tablet_tool *tev;
+	enum libinput_event_type type = LIBINPUT_EVENT_TABLET_TOOL_TIP;
+
+	litest_wait_for_event(li);
+	event = libinput_get_event(li);
+
+	litest_assert_notnull(event);
+	litest_assert_event_type(event, type);
+	tev = libinput_event_get_tablet_tool_event(event);
+	litest_assert_int_eq(libinput_event_tablet_tool_get_tip_state(tev),
+			     state);
+	libinput_event_destroy(event);
+}
+
 struct libinput_event_tablet_pad *
 litest_is_pad_button_event(struct libinput_event *event,
 			   unsigned int button,
@@ -3349,11 +3448,11 @@ litest_is_pad_button_event(struct libinput_event *event,
 	struct libinput_event_tablet_pad *p;
 	enum libinput_event_type type = LIBINPUT_EVENT_TABLET_PAD_BUTTON;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 
 	p = libinput_event_get_tablet_pad_event(event);
-	litest_assert(p != NULL);
+	litest_assert_ptr_notnull(p);
 
 	litest_assert_int_eq(libinput_event_tablet_pad_get_button_number(p),
 			     button);
@@ -3371,7 +3470,7 @@ litest_is_pad_ring_event(struct libinput_event *event,
 	struct libinput_event_tablet_pad *p;
 	enum libinput_event_type type = LIBINPUT_EVENT_TABLET_PAD_RING;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 	p = libinput_event_get_tablet_pad_event(event);
 
@@ -3391,7 +3490,7 @@ litest_is_pad_strip_event(struct libinput_event *event,
 	struct libinput_event_tablet_pad *p;
 	enum libinput_event_type type = LIBINPUT_EVENT_TABLET_PAD_STRIP;
 
-	litest_assert(event != NULL);
+	litest_assert_ptr_notnull(event);
 	litest_assert_event_type(event, type);
 	p = libinput_event_get_tablet_pad_event(event);
 
@@ -3449,7 +3548,7 @@ litest_assert_scroll(struct libinput *li,
 
 	event = libinput_get_event(li);
 	next_event = libinput_get_event(li);
-	litest_assert(next_event != NULL); /* At least 1 scroll + stop scroll */
+	litest_assert_ptr_notnull(next_event); /* At least 1 scroll + stop scroll */
 
 	while (event) {
 		ptrev = litest_is_axis_event(event, axis, 0);
@@ -3709,23 +3808,17 @@ litest_timeout_hysteresis(void)
 	msleep(90);
 }
 
-void
-litest_timeout_thumb(void)
-{
-	msleep(320);
-}
-
 void
 litest_push_event_frame(struct litest_device *dev)
 {
-	litest_assert(dev->skip_ev_syn >= 0);
+	litest_assert_int_ge(dev->skip_ev_syn, 0);
 	dev->skip_ev_syn++;
 }
 
 void
 litest_pop_event_frame(struct litest_device *dev)
 {
-	litest_assert(dev->skip_ev_syn > 0);
+	litest_assert_int_gt(dev->skip_ev_syn, 0);
 	dev->skip_ev_syn--;
 	if (dev->skip_ev_syn == 0)
 		litest_event(dev, EV_SYN, SYN_REPORT, 0);
@@ -3746,7 +3839,7 @@ litest_unfilter_event(struct litest_device *dev,
 {
 	/* would need an non-NULL argument for re-enabling, so simply abort
 	 * until we need to be more sophisticated */
-	litest_assert(type != EV_ABS);
+	litest_assert_int_ne(type, (unsigned int)EV_ABS);
 
 	libevdev_enable_event_code(dev->evdev, type, code, NULL);
 }
@@ -3927,6 +4020,7 @@ litest_parse_argv(int argc, char **argv)
 		JOBS_CUSTOM
 	} want_jobs = JOBS_DEFAULT;
 	char *builddir;
+	char *jobs_env;
 
 	/* If we are not running from the builddir, we assume we're running
 	 * against the system as installed */
@@ -3938,6 +4032,13 @@ litest_parse_argv(int argc, char **argv)
 	if (in_debugger)
 		want_jobs = JOBS_SINGLE;
 
+	if ((jobs_env = getenv("LITEST_JOBS"))) {
+		if (!safe_atoi(jobs_env, &jobs)) {
+			fprintf(stderr, "LITEST_JOBS environment variable must be positive integer\n");
+			exit(EXIT_FAILURE);
+		}
+	}
+
 	while(1) {
 		int c;
 		int option_index = 0;
@@ -3962,7 +4063,8 @@ litest_parse_argv(int argc, char **argv)
 			       "    --verbose\n"
 			       "          Enable verbose output\n"
 			       "    --jobs 8\n"
-			       "          Number of parallel test suites to run (default: 8)\n"
+			       "          Number of parallel test suites to run (default: 8).\n"
+			       "	  This overrides the LITEST_JOBS environment variable.\n"
 			       "    --list\n"
 			       "          List all tests\n"
 			       "\n"
@@ -3982,8 +4084,6 @@ litest_parse_argv(int argc, char **argv)
 			break;
 		case OPT_FILTER_GROUP:
 			filter_group = optarg;
-			if (want_jobs == JOBS_DEFAULT)
-				want_jobs = JOBS_SINGLE;
 			break;
 		case 'j':
 		case OPT_JOBS:
@@ -4149,21 +4249,15 @@ main(int argc, char **argv)
 	int rc;
 
 	in_debugger = is_debugger_attached();
-	if (in_debugger)
+	if (in_debugger || RUNNING_ON_VALGRIND)
 		setenv("CK_FORK", "no", 0);
 
 	mode = litest_parse_argv(argc, argv);
 	if (mode == LITEST_MODE_ERROR)
 		return EXIT_FAILURE;
 
-	/* You don't get to skip the deviceless tests */
-	if (!run_deviceless) {
-		if (getenv("SKIP_LIBINPUT_TEST_SUITE_RUNNER"))
-			return 77;
-
-		if ((rc = check_device_access()) != 0)
+	if (!run_deviceless && (rc = check_device_access()) != 0)
 			return rc;
-	}
 
 	litest_init_test_devices();
 
