@@ -78,24 +78,6 @@ struct test_collection {
 	}; \
 	static void (name##_setup)(void)
 
-extern void litest_setup_tests_udev(void);
-extern void litest_setup_tests_path(void);
-extern void litest_setup_tests_pointer(void);
-extern void litest_setup_tests_touch(void);
-extern void litest_setup_tests_log(void);
-extern void litest_setup_tests_tablet(void);
-extern void litest_setup_tests_pad(void);
-extern void litest_setup_tests_touchpad(void);
-extern void litest_setup_tests_touchpad_tap(void);
-extern void litest_setup_tests_touchpad_buttons(void);
-extern void litest_setup_tests_trackpoint(void);
-extern void litest_setup_tests_trackball(void);
-extern void litest_setup_tests_misc(void);
-extern void litest_setup_tests_keyboard(void);
-extern void litest_setup_tests_device(void);
-extern void litest_setup_tests_gestures(void);
-extern void litest_setup_tests_lid(void);
-
 void
 litest_fail_condition(const char *file,
 		      int line,
@@ -113,6 +95,15 @@ litest_fail_comparison_int(const char *file,
 			   const char *astr,
 			   const char *bstr);
 void
+litest_fail_comparison_double(const char *file,
+			      int line,
+			      const char *func,
+			      const char *operator,
+			      double a,
+			      double b,
+			      const char *astr,
+			      const char *bstr);
+void
 litest_fail_comparison_ptr(const char *file,
 			   int line,
 			   const char *func,
@@ -182,6 +173,17 @@ litest_fail_comparison_ptr(const char *file,
 						   #a_ " " #op_ " " #b_); \
 	} while(0)
 
+#define litest_assert_comparison_double_(a_, op_, b_) \
+	do { \
+		const double EPSILON = 1.0/256; \
+		__typeof__(a_) _a = a_; \
+		__typeof__(b_) _b = b_; \
+		if (!((_a) op_ (_b)) && fabs((_a) - (_b)) > EPSILON)  \
+			litest_fail_comparison_double(__FILE__, __LINE__, __func__,\
+						      #op_, _a, _b, \
+						      #a_, #b_); \
+	} while(0)
+
 #define litest_assert_ptr_eq(a_, b_) \
 	litest_assert_comparison_ptr_(a_, ==, b_)
 
@@ -195,22 +197,22 @@ litest_fail_comparison_ptr(const char *file,
 	litest_assert_comparison_ptr_(a_, !=, NULL)
 
 #define litest_assert_double_eq(a_, b_)\
-	ck_assert_int_eq((int)((a_) * 256), (int)((b_) * 256))
+	litest_assert_comparison_double_((a_), ==, (b_))
 
 #define litest_assert_double_ne(a_, b_)\
-	ck_assert_int_ne((int)((a_) * 256), (int)((b_) * 256))
+	litest_assert_comparison_double_((a_), !=, (b_))
 
 #define litest_assert_double_lt(a_, b_)\
-	ck_assert_int_lt((int)((a_) * 256), (int)((b_) * 256))
+	litest_assert_comparison_double_((a_), <, (b_))
 
 #define litest_assert_double_le(a_, b_)\
-	ck_assert_int_le((int)((a_) * 256), (int)((b_) * 256))
+	litest_assert_comparison_double_((a_), <=, (b_))
 
 #define litest_assert_double_gt(a_, b_)\
-	ck_assert_int_gt((int)((a_) * 256), (int)((b_) * 256))
+	litest_assert_comparison_double_((a_), >, (b_))
 
 #define litest_assert_double_ge(a_, b_)\
-	ck_assert_int_ge((int)((a_) * 256), (int)((b_) * 256))
+	litest_assert_comparison_double_((a_), >=, (b_))
 
 enum litest_device_type {
 	LITEST_NO_DEVICE = -1,
@@ -295,44 +297,46 @@ enum litest_device_type {
 	LITEST_AIPTEK,
 	LITEST_TOUCHSCREEN_INVALID_RANGE,
 	LITEST_TOUCHSCREEN_MT_TOOL_TYPE,
+	LITEST_DELL_CANVAS_TOTEM,
+	LITEST_DELL_CANVAS_TOTEM_TOUCH,
+	LITEST_WACOM_ISDV4_4200_PEN,
 };
 
-enum litest_device_feature {
-	LITEST_DEVICELESS	= -2,
-	LITEST_DISABLE_DEVICE	= -1,
-	LITEST_ANY		= 0,
-	LITEST_TOUCHPAD		= bit(0),
-	LITEST_CLICKPAD		= bit(1),
-	LITEST_BUTTON		= bit(2),
-	LITEST_KEYS		= bit(3),
-	LITEST_RELATIVE		= bit(4),
-	LITEST_WHEEL		= bit(5),
-	LITEST_TOUCH		= bit(6),
-	LITEST_SINGLE_TOUCH	= bit(7),
-	LITEST_APPLE_CLICKPAD	= bit(8),
-	LITEST_TOPBUTTONPAD	= bit(9),
-	LITEST_SEMI_MT		= bit(10),
-	LITEST_POINTINGSTICK	= bit(11),
-	LITEST_FAKE_MT		= bit(12),
-	LITEST_ABSOLUTE		= bit(13),
-	LITEST_PROTOCOL_A	= bit(14),
-	LITEST_HOVER		= bit(15),
-	LITEST_ELLIPSE		= bit(16),
-	LITEST_TABLET		= bit(17),
-	LITEST_DISTANCE		= bit(18),
-	LITEST_TOOL_SERIAL	= bit(19),
-	LITEST_TILT		= bit(20),
-	LITEST_TABLET_PAD	= bit(21),
-	LITEST_RING		= bit(22),
-	LITEST_STRIP		= bit(23),
-	LITEST_TRACKBALL	= bit(24),
-	LITEST_LEDS		= bit(25),
-	LITEST_SWITCH		= bit(26),
-	LITEST_IGNORED		= bit(27),
-	LITEST_NO_DEBOUNCE	= bit(28),
-	LITEST_TOOL_MOUSE	= bit(29),
-	LITEST_DIRECT		= bit(30),
-};
+#define LITEST_DEVICELESS	-2
+#define LITEST_DISABLE_DEVICE	-1
+#define LITEST_ANY		0
+#define LITEST_TOUCHPAD		bit(0)
+#define LITEST_CLICKPAD		bit(1)
+#define LITEST_BUTTON		bit(2)
+#define LITEST_KEYS		bit(3)
+#define LITEST_RELATIVE		bit(4)
+#define LITEST_WHEEL		bit(5)
+#define LITEST_TOUCH		bit(6)
+#define LITEST_SINGLE_TOUCH	bit(7)
+#define LITEST_APPLE_CLICKPAD	bit(8)
+#define LITEST_TOPBUTTONPAD	bit(9)
+#define LITEST_SEMI_MT		bit(10)
+#define LITEST_POINTINGSTICK	bit(11)
+#define LITEST_FAKE_MT		bit(12)
+#define LITEST_ABSOLUTE		bit(13)
+#define LITEST_PROTOCOL_A	bit(14)
+#define LITEST_HOVER		bit(15)
+#define LITEST_ELLIPSE		bit(16)
+#define LITEST_TABLET		bit(17)
+#define LITEST_DISTANCE		bit(18)
+#define LITEST_TOOL_SERIAL	bit(19)
+#define LITEST_TILT		bit(20)
+#define LITEST_TABLET_PAD	bit(21)
+#define LITEST_RING		bit(22)
+#define LITEST_STRIP		bit(23)
+#define LITEST_TRACKBALL	bit(24)
+#define LITEST_LEDS		bit(25)
+#define LITEST_SWITCH		bit(26)
+#define LITEST_IGNORED		bit(27)
+#define LITEST_NO_DEBOUNCE	bit(28)
+#define LITEST_TOOL_MOUSE	bit(29)
+#define LITEST_DIRECT		bit(30)
+#define LITEST_TOTEM		bit(31)
 
 /* this is a semi-mt device, so we keep track of the touches that the tests
  * send and modify them so that the first touch is always slot 0 and sends
@@ -434,14 +438,14 @@ void
 _litest_add(const char *name,
 	    const char *funcname,
 	    void *func,
-	    enum litest_device_feature required_feature,
-	    enum litest_device_feature excluded_feature);
+	    int64_t required_feature,
+	    int64_t excluded_feature);
 void
 _litest_add_ranged(const char *name,
 		   const char *funcname,
 		   void *func,
-		   enum litest_device_feature required,
-		   enum litest_device_feature excluded,
+		   int64_t required,
+		   int64_t excluded,
 		   const struct range *range);
 void
 _litest_add_for_device(const char *name,
@@ -776,6 +780,10 @@ void
 litest_assert_tablet_proximity_event(struct libinput *li,
 				     enum libinput_tablet_tool_proximity_state state);
 
+void
+litest_assert_tablet_tip_event(struct libinput *li,
+			       enum libinput_tablet_tool_tip_state state);
+
 void
 litest_assert_pad_button_event(struct libinput *li,
 				    unsigned int button,
@@ -839,9 +847,6 @@ litest_timeout_touch_arbitration(void);
 void
 litest_timeout_hysteresis(void);
 
-void
-litest_timeout_thumb(void);
-
 void
 litest_push_event_frame(struct litest_device *dev);
 
